@@ -16,6 +16,7 @@ import re
 import unittest
 
 from oslo_config import types
+from six.moves import range as compat_range
 
 
 class ConfigTypeTests(unittest.TestCase):
@@ -62,6 +63,10 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         self.type_instance = types.String(choices=['foo', 'bar'])
         self.assertConvertedValue('foo', 'foo')
 
+    def test_listed_value_tuple(self):
+        self.type_instance = types.String(choices=('foo', 'bar'))
+        self.assertConvertedValue('foo', 'foo')
+
     def test_unlisted_value(self):
         self.type_instance = types.String(choices=['foo', 'bar'])
         self.assertInvalid('baz')
@@ -91,6 +96,10 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.String(choices=['foo', 'bar'])
         self.assertEqual('String(choices=[\'foo\', \'bar\'])', repr(t))
 
+    def test_repr_with_choices_tuple(self):
+        t = types.String(choices=('foo', 'bar'))
+        self.assertEqual('String(choices=(\'foo\', \'bar\'))', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.String() == types.String())
 
@@ -106,7 +115,9 @@ class StringTypeTests(TypeTestHelper, unittest.TestCase):
     def test_not_equal_with_different_choices(self):
         t1 = types.String(choices=['foo', 'bar'])
         t2 = types.String(choices=['foo', 'baz'])
+        t3 = types.String(choices=('foo', 'baz'))
         self.assertFalse(t1 == t2)
+        self.assertFalse(t1 == t3)
 
     def test_equal_with_equal_quote_falgs(self):
         t1 = types.String(quotes=True)
@@ -269,6 +280,10 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
         t = types.Integer(choices=[80, 457])
         self.assertEqual('Integer(choices=[80, 457])', repr(t))
 
+    def test_repr_with_choices_tuple(self):
+        t = types.Integer(choices=(80, 457))
+        self.assertEqual('Integer(choices=(80, 457))', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.Integer() == types.Integer())
 
@@ -286,7 +301,9 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
     def test_equal_with_same_choices(self):
         t1 = types.Integer(choices=[80, 457])
         t2 = types.Integer(choices=[457, 80])
+        t3 = types.Integer(choices=(457, 80))
         self.assertTrue(t1 == t2)
+        self.assertTrue(t1 == t3)
 
     def test_not_equal(self):
         self.assertFalse(types.Integer(min=123) == types.Integer(min=456))
@@ -301,16 +318,13 @@ class IntegerTypeTests(TypeTestHelper, unittest.TestCase):
     def test_choices_with_min_max(self):
         self.assertRaises(ValueError,
                           types.Integer,
-                          min=10,
+                          min=100,
                           choices=[50, 60])
         self.assertRaises(ValueError,
                           types.Integer,
-                          max=100,
-                          choices=[50, 60])
-        self.assertRaises(ValueError,
-                          types.Integer,
-                          min=10, max=100,
+                          max=10,
                           choices=[50, 60])
+        types.Integer(min=10, max=100, choices=[50, 60])
 
     def test_min_greater_max(self):
         self.assertRaises(ValueError,
@@ -394,12 +408,90 @@ class FloatTypeTests(TypeTestHelper, unittest.TestCase):
     def test_repr(self):
         self.assertEqual('Float', repr(types.Float()))
 
+    def test_repr_with_min(self):
+        t = types.Float(min=1.1)
+        self.assertEqual('Float(min=1.1)', repr(t))
+
+    def test_repr_with_max(self):
+        t = types.Float(max=2.2)
+        self.assertEqual('Float(max=2.2)', repr(t))
+
+    def test_repr_with_min_and_max(self):
+        t = types.Float(min=1.1, max=2.2)
+        self.assertEqual('Float(min=1.1, max=2.2)', repr(t))
+        t = types.Float(min=1.0, max=2)
+        self.assertEqual('Float(min=1, max=2)', repr(t))
+        t = types.Float(min=0, max=0)
+        self.assertEqual('Float(min=0, max=0)', repr(t))
+
     def test_equal(self):
         self.assertTrue(types.Float() == types.Float())
 
+    def test_equal_with_same_min_and_no_max(self):
+        self.assertTrue(types.Float(min=123.1) == types.Float(min=123.1))
+
+    def test_equal_with_same_max_and_no_min(self):
+        self.assertTrue(types.Float(max=123.1) == types.Float(max=123.1))
+
+    def test_not_equal(self):
+        self.assertFalse(types.Float(min=123.1) == types.Float(min=456.1))
+        self.assertFalse(types.Float(max=123.1) == types.Float(max=456.1))
+        self.assertFalse(types.Float(min=123.1) == types.Float(max=123.1))
+        self.assertFalse(types.Float(min=123.1, max=456.1) ==
+                         types.Float(min=123.1, max=456.2))
+
     def test_not_equal_to_other_class(self):
         self.assertFalse(types.Float() == types.Integer())
 
+    def test_equal_with_same_min_and_max(self):
+        t1 = types.Float(min=1.1, max=2.2)
+        t2 = types.Float(min=1.1, max=2.2)
+        self.assertTrue(t1 == t2)
+
+    def test_min_greater_max(self):
+        self.assertRaises(ValueError,
+                          types.Float,
+                          min=100.1, max=50)
+        self.assertRaises(ValueError,
+                          types.Float,
+                          min=-50, max=-100.1)
+        self.assertRaises(ValueError,
+                          types.Float,
+                          min=0.1, max=-50.0)
+        self.assertRaises(ValueError,
+                          types.Float,
+                          min=50.0, max=0.0)
+
+    def test_with_max_and_min(self):
+        t = types.Float(min=123.45, max=678.9)
+        self.assertRaises(ValueError, t, 123)
+        self.assertRaises(ValueError, t, 123.1)
+        t(124.1)
+        t(300)
+        t(456.0)
+        self.assertRaises(ValueError, t, 0)
+        self.assertRaises(ValueError, t, 800.5)
+
+    def test_with_min_zero(self):
+        t = types.Float(min=0, max=456.1)
+        self.assertRaises(ValueError, t, -1)
+        t(0.0)
+        t(123.1)
+        t(300.2)
+        t(456.1)
+        self.assertRaises(ValueError, t, -201.0)
+        self.assertRaises(ValueError, t, 457.0)
+
+    def test_with_max_zero(self):
+        t = types.Float(min=-456.1, max=0)
+        self.assertRaises(ValueError, t, 1)
+        t(0.0)
+        t(-123.0)
+        t(-300.0)
+        t(-456.0)
+        self.assertRaises(ValueError, t, 201.0)
+        self.assertRaises(ValueError, t, -457.0)
+
 
 class ListTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.List()
@@ -467,6 +559,45 @@ class ListTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertFalse(types.List() == types.Integer())
 
 
+class RangeTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.Range()
+
+    def assertRange(self, s, r1, r2, step=1):
+        self.assertEqual(list(compat_range(r1, r2, step)),
+                         list(self.type_instance(s)))
+
+    def test_range(self):
+        self.assertRange('0-2', 0, 3)
+        self.assertRange('-2-0', -2, 1)
+        self.assertRange('2-0', 2, -1, -1)
+        self.assertRange('-3--1', -3, 0)
+        self.assertRange('-1--3', -1, -4, -1)
+        self.assertRange('-1', -1, 0)
+        self.assertInvalid('--1')
+        self.assertInvalid('4-')
+        self.assertInvalid('--')
+        self.assertInvalid('1.1-1.2')
+        self.assertInvalid('a-b')
+
+    def test_range_bounds(self):
+        self.type_instance = types.Range(1, 3)
+        self.assertRange('1-3', 1, 4)
+        self.assertRange('2-2', 2, 3)
+        self.assertRange('2', 2, 3)
+        self.assertInvalid('1-4')
+        self.assertInvalid('0-3')
+        self.assertInvalid('0-4')
+
+    def test_range_exclusive(self):
+        self.type_instance = types.Range(inclusive=False)
+        self.assertRange('0-2', 0, 2)
+        self.assertRange('-2-0', -2, 0)
+        self.assertRange('2-0', 2, 0, -1)
+        self.assertRange('-3--1', -3, -1)
+        self.assertRange('-1--3', -1, -3, -1)
+        self.assertRange('-1', -1, -1)
+
+
 class DictTypeTests(TypeTestHelper, unittest.TestCase):
     type = types.Dict()
 
@@ -637,3 +768,179 @@ class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
         self.assertEqual(255, len(test_str))
         self.assertInvalid(test_str)
         self.assertConvertedEqual(test_str[:-2])
+
+
+class URITypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.URI()
+
+    def test_uri(self):
+        self.assertConvertedValue('http://example.com', 'http://example.com')
+        self.assertInvalid('invalid')  # it doesn't include a scheme
+        self.assertInvalid('http://')  # it doesn't include an authority
+
+    def test_repr(self):
+        self.assertEqual('URI', repr(types.URI()))
+
+    def test_max_length(self):
+        self.type_instance = types.String(max_length=30)
+        self.assertInvalid('http://www.example.com/versions')
+        self.assertConvertedValue('http://www.example.com',
+                                  'http://www.example.com')
+
+
+class PortTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.Port()
+
+    def test_port(self):
+        self.assertInvalid(-1)
+        self.assertInvalid(65536)
+        self.assertConvertedValue('80', 80)
+        self.assertConvertedValue('65535', 65535)
+
+    def test_repr(self):
+        self.assertEqual('Port(min=0, max=65535)', repr(types.Port()))
+
+    def test_repr_with_min(self):
+        t = types.Port(min=123)
+        self.assertEqual('Port(min=123, max=65535)', repr(t))
+
+    def test_repr_with_max(self):
+        t = types.Port(max=456)
+        self.assertEqual('Port(min=0, max=456)', repr(t))
+
+    def test_repr_with_min_and_max(self):
+        t = types.Port(min=123, max=456)
+        self.assertEqual('Port(min=123, max=456)', repr(t))
+        t = types.Port(min=0, max=0)
+        self.assertEqual('Port(min=0, max=0)', repr(t))
+
+    def test_repr_with_choices(self):
+        t = types.Port(choices=[80, 457])
+        self.assertEqual('Port(choices=[80, 457])', repr(t))
+
+    def test_repr_with_choices_tuple(self):
+        t = types.Port(choices=(80, 457))
+        self.assertEqual('Port(choices=(80, 457))', repr(t))
+
+    def test_choices(self):
+        t = types.Port(choices=[80, 457])
+        self.assertRaises(ValueError, t, 1)
+        self.assertRaises(ValueError, t, 200)
+        t(80)
+        t(457)
+
+    def test_invalid_choices(self):
+        self.assertRaises(ValueError, types.Port, choices=[-1, 457])
+        self.assertRaises(ValueError, types.Port, choices=[1, 2, 3, 65536])
+
+    def test_equal(self):
+        self.assertTrue(types.Port() == types.Port())
+
+    def test_equal_with_same_min_and_no_max(self):
+        self.assertTrue(types.Port(min=123) == types.Port(min=123))
+
+    def test_equal_with_same_max_and_no_min(self):
+        self.assertTrue(types.Port(max=123) == types.Port(max=123))
+
+    def test_equal_with_same_min_and_max(self):
+        t1 = types.Port(min=1, max=123)
+        t2 = types.Port(min=1, max=123)
+        self.assertTrue(t1 == t2)
+
+    def test_equal_with_same_choices(self):
+        t1 = types.Port(choices=[80, 457])
+        t2 = types.Port(choices=[457, 80])
+        t3 = types.Port(choices=(457, 80))
+        self.assertTrue(t1 == t2)
+        self.assertTrue(t1 == t3)
+
+    def test_not_equal(self):
+        self.assertFalse(types.Port(min=123) == types.Port(min=456))
+        self.assertFalse(types.Port(choices=[80, 457]) ==
+                         types.Port(choices=[80, 40]))
+        self.assertFalse(types.Port(choices=[80, 457]) ==
+                         types.Port())
+
+    def test_not_equal_to_other_class(self):
+        self.assertFalse(types.Port() == types.Integer())
+
+    def test_choices_with_min_max(self):
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=100,
+                          choices=[50, 60])
+        self.assertRaises(ValueError,
+                          types.Port,
+                          max=10,
+                          choices=[50, 60])
+        types.Port(min=10, max=100, choices=[50, 60])
+
+    def test_min_greater_max(self):
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=100, max=50)
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=-50, max=-100)
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=0, max=-50)
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=50, max=0)
+
+    def test_illegal_min(self):
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=-1, max=50)
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=-50)
+
+    def test_illegal_max(self):
+        self.assertRaises(ValueError,
+                          types.Port,
+                          min=100, max=65537)
+        self.assertRaises(ValueError,
+                          types.Port,
+                          max=100000)
+
+    def test_with_max_and_min(self):
+        t = types.Port(min=123, max=456)
+        self.assertRaises(ValueError, t, 122)
+        t(123)
+        t(300)
+        t(456)
+        self.assertRaises(ValueError, t, 0)
+        self.assertRaises(ValueError, t, 457)
+
+    def test_with_min_zero(self):
+        t = types.Port(min=0, max=456)
+        self.assertRaises(ValueError, t, -1)
+        t(0)
+        t(123)
+        t(300)
+        t(456)
+        self.assertRaises(ValueError, t, -201)
+        self.assertRaises(ValueError, t, 457)
+
+    def test_with_max_zero(self):
+        t = types.Port(max=0)
+        self.assertRaises(ValueError, t, 1)
+        t(0)
+
+    def test_with_choices_list(self):
+        t = types.Port(choices=[80, 457])
+        self.assertRaises(ValueError, t, 1)
+        self.assertRaises(ValueError, t, 200)
+        self.assertRaises(ValueError, t, -457)
+        t(80)
+        t(457)
+
+    def test_with_choices_tuple(self):
+        t = types.Port(choices=(80, 457))
+        self.assertRaises(ValueError, t, 1)
+        self.assertRaises(ValueError, t, 200)
+        self.assertRaises(ValueError, t, -457)
+        t(80)
+        t(457)
